"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _mapper = require("../mapper");

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var buildXhrResponse = function buildXhrResponse(xhr) {
  return {
    data: xhr.response,
    status: xhr.status,
    statusText: xhr.statusText,
    ok: xhr.status >= 200 && xhr.status < 300,
    responseHeaders: xhr.getAllResponseHeaders(),
    getResponseHeader: xhr.getResponseHeader
  };
};

var REQUEST_STATE_SUCCESS = "REQUEST_STATE_SUCCESS";
var REQUEST_STATE_ERROR = "REQUEST_STATE_ERROR";
var REQUEST_STATE_ABORTED = "REQUEST_STATE_ABORTED";

var ObservableRequest = function () {
  function ObservableRequest() {
    _classCallCheck(this, ObservableRequest);

    this.listeners = [];
    this.state = null;
    this.data = null;
  }

  _createClass(ObservableRequest, [{
    key: "subscribe",
    value: function subscribe(listener, state) {
      this.listeners = [].concat(_toConsumableArray(this.listeners), [{ listener: listener, state: state }]);
      if (this.state !== null && state === this.state) listener(this.data);
    }
  }, {
    key: "setState",
    value: function setState(reqState, data) {
      var _this = this;

      this.state = reqState;
      this.data = data;

      this.listeners.forEach(function (_ref) {
        var listener = _ref.listener,
            state = _ref.state;

        if (state === _this.state) listener(_this.data);
      });
    }
  }]);

  return ObservableRequest;
}();

;

var XhrHttpLayer = function () {
  function XhrHttpLayer() {
    _classCallCheck(this, XhrHttpLayer);

    this.requestsContainer = [];
  }

  _createClass(XhrHttpLayer, [{
    key: "addRequest",
    value: function addRequest(xhr, endPoint) {
      var req = { endPoint: endPoint, xhr: xhr };
      this.requestsContainer = [].concat(_toConsumableArray(this.requestsContainer), [req]);

      return req;
    }
  }, {
    key: "addMemRequest",
    value: function addMemRequest(timeoutId, endPoint, observableRequest) {
      var req = {
        endPoint: endPoint,
        xhr: {
          abort: function abort() {
            clearTimeout(timeoutId);
            observableRequest.setState(REQUEST_STATE_ABORTED);
          }
        }
      };

      this.requestsContainer = [].concat(_toConsumableArray(this.requestsContainer), [req]);

      return req;
    }
  }, {
    key: "handleRequest",
    value: function handleRequest(stateDispatcher, request, method) {
      var _this2 = this;

      var options = request.options;
      var beforeRequest = options.beforeRequest || function () {};
      var afterResponse = options.afterResponse || function () {};
      var afterError = options.afterError || function () {};
      var afterAbort = options.afterAbort || function () {};
      var parseResponse = options.parseResponse || options.responseParse || function (r) {
        return r.data;
      };
      var parseBody = options.parseBody || options.bodyParse || function (b) {
        return b ? JSON.stringify(b) : b;
      };
      var payload = options.payload || {};

      var memResponse = options.memResponse;
      var delay = options.delay;
      var cacheHandler = options.cacheHandler;

      var responseType = options.responseType || "json";
      var mimeType = options.mimeType;

      var observableRequest = new ObservableRequest();
      var currentRequest = null;

      //memory response
      if (memResponse !== undefined) {
        var memResponseFn = memResponse;

        beforeRequest(request);
        stateDispatcher(_mapper.FetchStates.FETCH_STARTED, payload);

        if (typeof memResponse !== "function") memResponseFn = function memResponseFn() {
          return memResponse;
        };

        if (delay > 0) {
          var timeoutId = setTimeout(function () {
            var response = memResponseFn(request);
            stateDispatcher(_mapper.FetchStates.FETCH_COMPLETED, _extends({}, payload, { data: response }));
            afterResponse(request, response);

            observableRequest.setState(REQUEST_STATE_SUCCESS, response);
            _this2.removeRequest(currentRequest);
          }, delay);

          currentRequest = this.addMemRequest(timeoutId, request.endPoint, observableRequest);
        } else {
          var response = memResponseFn(request);
          stateDispatcher(_mapper.FetchStates.FETCH_COMPLETED, _extends({}, payload, { data: response }));
          afterResponse(request, response);

          observableRequest.setState(REQUEST_STATE_SUCCESS, response);
        }
      } else {
        beforeRequest(request);
        stateDispatcher(_mapper.FetchStates.FETCH_STARTED, payload);
        var cacheResponse = void 0;

        if (cacheHandler) cacheResponse = cacheHandler(request);

        if (cacheResponse !== undefined) {
          stateDispatcher(_mapper.FetchStates.FETCH_COMPLETED, _extends({}, payload, { data: cacheResponse }));
          afterResponse(request, cacheResponse);

          observableRequest.setState(REQUEST_STATE_SUCCESS, _extends({}, payload, { data: cacheResponse }));
        } else {
          var xhr = new XMLHttpRequest();
          currentRequest = this.addRequest(xhr, request.endPoint);

          var handleLoad = function handleLoad() {
            var response = buildXhrResponse(xhr);
            var parsedResponse = parseResponse(response);

            if (response.ok) stateDispatcher(_mapper.FetchStates.FETCH_COMPLETED, _extends({}, payload, { data: parsedResponse }));else stateDispatcher(_mapper.FetchStates.FETCH_ERROR, _extends({}, payload, { data: parsedResponse }));

            afterResponse(request, parsedResponse);

            if (response.ok) observableRequest.setState(REQUEST_STATE_SUCCESS, parsedResponse);else observableRequest.setState(REQUEST_STATE_ERROR, parsedResponse);

            _this2.removeRequest(currentRequest);
          };

          var handleError = function handleError() {
            var parsedResponse = parseResponse(buildXhrResponse(xhr));
            stateDispatcher(_mapper.FetchStates.FETCH_ERROR, _extends({}, payload, { data: parsedResponse }));
            afterError(request, parsedResponse);

            observableRequest.setState(REQUEST_STATE_ERROR, parsedResponse);
            _this2.removeRequest(currentRequest);
          };

          var handleAbort = function handleAbort() {
            stateDispatcher(_mapper.FetchStates.FETCH_CANCELLED, payload);
            afterAbort(request);

            observableRequest.setState(REQUEST_STATE_ABORTED);
            _this2.removeRequest(currentRequest);
          };

          xhr.addEventListener('load', handleLoad);
          xhr.addEventListener('error', handleError);
          xhr.addEventListener('abort', handleAbort);

          xhr.open(method, request.fullPath);

          for (var header in request.headers) {
            xhr.setRequestHeader(header, request.headers[header]);
          }xhr.responseType = responseType;

          if (mimeType) xhr.overrideMimeType(mimeType);

          xhr.send(parseBody(request.body));
        }
      }

      return function (cbSuccess, cbError, cbAbort) {
        if (cbSuccess) observableRequest.subscribe(cbSuccess, REQUEST_STATE_SUCCESS);

        if (cbError) observableRequest.subscribe(cbError, REQUEST_STATE_ERROR);

        if (cbAbort) observableRequest.subscribe(cbAbort, REQUEST_STATE_ABORTED);
      };
    }
  }, {
    key: "abortRequest",
    value: function abortRequest(endPoint) {
      var aborted = false;

      this.requestsContainer = this.requestsContainer.filter(function (req) {
        if (req.endPoint === endPoint) {
          req.xhr.abort();
          aborted = true;
          return false;
        }
        return true;
      });

      return aborted;
    }
  }, {
    key: "removeRequest",
    value: function removeRequest(r) {
      this.requestsContainer = this.requestsContainer.filter(function (req) {
        return req === r;
      });
    }
  }, {
    key: "get",
    value: function get(stateDispatcher, request) {
      return this.handleRequest(stateDispatcher, request, _mapper.HttpMethods.GET);
    }
  }, {
    key: "post",
    value: function post(stateDispatcher, request) {
      return this.handleRequest(stateDispatcher, request, _mapper.HttpMethods.POST);
    }
  }, {
    key: "put",
    value: function put(stateDispatcher, request) {
      return this.handleRequest(stateDispatcher, request, _mapper.HttpMethods.PUT);
    }
  }, {
    key: "delete",
    value: function _delete(stateDispatcher, request) {
      return this.handleRequest(stateDispatcher, request, _mapper.HttpMethods.DELETE);
    }
  }, {
    key: "patch",
    value: function patch(stateDispatcher, request) {
      return this.handleRequest(stateDispatcher, request, _mapper.HttpMethods.PATCH);
    }
  }, {
    key: "head",
    value: function head(stateDispatcher, request) {
      return this.handleRequest(stateDispatcher, request, _mapper.HttpMethods.HEAD);
    }
  }]);

  return XhrHttpLayer;
}();

exports.default = XhrHttpLayer;