'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _actionBuilder = require('../reducer/action-builder');

var _actionBuilder2 = _interopRequireDefault(_actionBuilder);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Build a state dispatcher
 * @param store
 * @param stateToAction
 * @param mEndPoint
 * @returns {stateDispatcher}
 */
function buildStateDispatcher(store, stateToAction, mEndPoint) {
  var storeDispatcher = store.dispatch;
  var mapper = mEndPoint._resource._mapper;
  var resource = mEndPoint._resource;
  var actionBuilder = (0, _actionBuilder2.default)(resource.name, mEndPoint.name, mapper.name);

  /**
   * Dispatch an action according to the state of the request to the redux store. If any state handler is provided, than it will
   * be called according to the state.
   * @param {FetchStates} state - The state of the request
   * @param {Object} [payload] - The payload to be dispatched
   */
  var stateDispatcher = function stateDispatcher(state, payload) {

    //Dispatch actions defined by the stateToAction
    if (stateToAction) {
      var actionToDispatch = stateToAction(state);
      if (actionToDispatch) handleActionToDispatch(mapper, storeDispatcher, actionToDispatch, payload);
    }

    //Dispatch action to the store
    var actionType = actionBuilder(state);

    var action = {
      type: actionType,
      payload: payload
    };

    storeDispatcher(action);
  };

  return stateDispatcher;
}

function handleActionToDispatch(mapper, storeDispatcher, actionToDispatch, payload) {
  //no action to dispatch
  if (actionToDispatch === null || actionToDispatch === undefined) return;

  //Multiple actions to dispatch
  if (Array.isArray(actionToDispatch)) {
    actionToDispatch.forEach(function (action) {
      return handleActionToDispatch(mapper, storeDispatcher, action, payload);
    });
    return;
  }

  var fnResult = void 0;

  //Function
  if (typeof actionToDispatch === 'function') {
    fnResult = actionToDispatch(payload, mapper, storeDispatcher);

    //no need to dispatch
    if (fnResult === null || fnResult === undefined) return;

    handleActionToDispatch(mapper, storeDispatcher, fnResult, payload);
    return;
  }

  //Object
  if ((typeof actionToDispatch === 'undefined' ? 'undefined' : _typeof(actionToDispatch)) === 'object') {
    storeDispatcher(actionToDispatch);
    return;
  }

  throw new Error('Cannot handle ' + (typeof actionToDispatch === 'undefined' ? 'undefined' : _typeof(actionToDispatch)) + '. Invalid action dispatcher handler');
}

exports.default = buildStateDispatcher;