'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _redux = require('redux');

var _actionBuilder = require('./action-builder');

var _actionBuilder2 = _interopRequireDefault(_actionBuilder);

var _defaultReducer = require('./default-reducer');

var _defaultReducer2 = _interopRequireDefault(_defaultReducer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Generates a combined reducer for all api end-points based on the configuration provided
 * @param {Object} config - The configuration object
 * @returns {Reducer<S>}
 */
function createApiReducer(config) {
  var resourcesReducers = config.resources.reduce(function (acc, resource) {
    var name = resource.reducerName || resource.name;
    acc[name] = createResourceReducer(config, resource);
    return acc;
  }, {});

  return (0, _redux.combineReducers)(resourcesReducers);
}

function createResourceReducer(config, resource) {
  var endPointsReducers = resource.endPoints.reduce(function (acc, endPoint) {
    var actionBuilder = (0, _actionBuilder2.default)(resource.name, endPoint.name, config.name);
    var name = endPoint.reducerName || endPoint.name;

    var reducerData = void 0;

    if (endPoint.reducerData !== undefined) reducerData = endPoint.reducerData;else if (resource.reducerData !== undefined) reducerData = resource.reducerData;else if (config.reducerData !== undefined) reducerData = config.reducerData;else reducerData = undefined;

    if (endPoint.reducerBuilder) acc[name] = endPoint.reducerBuilder(actionBuilder, reducerData);else if (resource.reducerBuilder) acc[name] = resource.reducerBuilder(actionBuilder, reducerData);else if (config.reducerBuilder) acc[name] = config.reducerBuilder(actionBuilder, reducerData);else acc[name] = (0, _defaultReducer2.default)(actionBuilder, reducerData);

    return acc;
  }, {});

  return (0, _redux.combineReducers)(endPointsReducers);
};

exports.default = createApiReducer;